@@ -1,7 +1,7 @@
 /*
  *   mooeditprefspage.c
  *
- *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
  *   This file is part of medit.  medit is free software; you can
  *   redistribute it and/or modify it under the terms of the
@@ -15,7 +15,7 @@
 
 #define MOOEDIT_COMPILATION
 #include "mooedit/mooedit-impl.h"
-#include "mooedit/mooeditor-private.h"
+#include "mooedit/mooeditor-impl.h"
 #include "mooedit/mooeditprefs.h"
 #include "mooedit/moolangmgr.h"
 #include "mooedit/mooeditfiltersettings.h"
@@ -28,12 +28,12 @@
 #include "mooutils/mooencodings.h"
 #include "mooutils/mooi18n.h"
 #include "mooutils/moohelp.h"
-#include "glade/mooeditprefs-file-gxml.h"
-#include "glade/mooeditprefs-filters-gxml.h"
-#include "glade/mooeditprefs-general-gxml.h"
-#include "glade/mooeditprefs-langs-gxml.h"
-#include "glade/mooeditprefs-view-gxml.h"
-#include "help-sections.h"
+#include "mooedit/mooeditprefs-file-gxml.h"
+#include "mooedit/mooeditprefs-filters-gxml.h"
+#include "mooedit/mooeditprefs-general-gxml.h"
+#include "mooedit/mooeditprefs-langs-gxml.h"
+#include "mooedit/mooeditprefs-view-gxml.h"
+#include "moo-help-sections.h"
 #include <string.h>
 
 
@@ -51,11 +51,6 @@ static void     scheme_combo_data_func      (GtkCellLayout      *layout,
 static void     scheme_combo_set_scheme     (GtkComboBox        *combo,
                                              MooTextStyleScheme *scheme);
 
-static void     default_lang_combo_init     (GtkComboBox        *combo,
-                                             MooPrefsPage       *page);
-static void     default_lang_combo_set_lang (GtkComboBox        *combo,
-                                             const char         *id);
-
 static void     lang_combo_init             (GtkComboBox        *combo,
                                              MooPrefsPage       *page,
                                              PrefsLangsXml      *gxml);
@@ -69,37 +64,6 @@ static void     save_encoding_combo_apply   (PrefsFileXml       *gxml);
 
 static GtkTreeModel *page_get_lang_model    (MooPrefsPage       *page);
 static MooTextStyleScheme *page_get_scheme  (PrefsGeneralXml    *gxml);
-static char    *page_get_default_lang       (PrefsLangsXml      *gxml);
-
-
-static void
-add_page (GtkWidget *page,
-          GtkWidget *notebook,
-          GtkWidget *subpage)
-{
-    GtkWidget *label = gtk_label_new (MOO_PREFS_PAGE (subpage)->label);
-    gtk_widget_show (subpage);
-    gtk_notebook_append_page (GTK_NOTEBOOK (notebook), subpage, label);
-    moo_prefs_page_add_page (MOO_PREFS_PAGE (page), MOO_PREFS_PAGE (subpage));
-}
-
-GtkWidget *
-moo_edit_prefs_page_new (MooEditor *editor)
-{
-    GtkWidget *page, *notebook;
-
-    page = moo_prefs_page_new ("Editor", GTK_STOCK_EDIT);
-    notebook = gtk_notebook_new ();
-    gtk_container_add (GTK_CONTAINER (page), notebook);
-    gtk_widget_show_all (page);
-
-    add_page (page, notebook, moo_edit_prefs_page_new_1 (editor));
-    add_page (page, notebook, moo_edit_prefs_page_new_2 (editor));
-    add_page (page, notebook, moo_edit_prefs_page_new_3 (editor));
-    add_page (page, notebook, moo_edit_prefs_page_new_4 (editor));
-
-    return page;
-}
 
 
 static GtkWidget *
@@ -121,7 +85,7 @@ prefs_page_new (MooEditor          *editor,
                                   init_ui, init, apply);
 
     g_signal_connect_swapped (prefs_page, "apply",
-                              G_CALLBACK (moo_editor_queue_apply_prefs),
+                              G_CALLBACK (_moo_editor_apply_prefs),
                               editor);
 
     return prefs_page;
@@ -149,6 +113,8 @@ page_general_init_ui (MooPrefsPage *page)
     BIND_SETTING (tab_width, MOO_EDIT_PREFS_TAB_WIDTH);
     BIND_SETTING (indent_width, MOO_EDIT_PREFS_INDENT_WIDTH);
     BIND_SETTING (fontbutton, MOO_EDIT_PREFS_FONT);
+
+    moo_help_set_id (GTK_WIDGET (page), HELP_SECTION_PREFS_GENERAL);
 }
 
 static void
@@ -199,7 +165,6 @@ GtkWidget *
 moo_edit_prefs_page_new_1 (MooEditor *editor)
 {
     return prefs_page_new (editor,
-                           /* Translators: this one is temporary, don't translate */
                            Q_("PreferencesPage|General"),
                            GTK_STOCK_EDIT,
                            page_general_init_ui,
@@ -214,6 +179,7 @@ page_filters_init_ui (MooPrefsPage *page)
     PrefsFiltersXml *gxml;
     gxml = prefs_filters_xml_new_with_root (GTK_WIDGET (page));
     g_object_set_data (G_OBJECT (page), "moo-edit-prefs-page-xml", gxml);
+    moo_help_set_id (GTK_WIDGET (page), HELP_SECTION_PREFS_FILTERS);
 }
 
 static void
@@ -234,8 +200,7 @@ GtkWidget *
 moo_edit_prefs_page_new_5 (MooEditor *editor)
 {
     return prefs_page_new (editor,
-                           /* Translators: this one is temporary, don't translate */
-                           Q_("PreferencesPage|Filters"),
+                           Q_("PreferencesPage|File Filters"),
                            GTK_STOCK_EDIT,
                            page_filters_init_ui,
                            page_filters_init,
@@ -261,9 +226,12 @@ page_view_init_ui (MooPrefsPage *page)
     BIND_SETTING (check_highlight_current_line, MOO_EDIT_PREFS_HIGHLIGHT_CURRENT_LINE);
     BIND_SETTING (show_line_numbers, MOO_EDIT_PREFS_SHOW_LINE_NUMBERS);
     BIND_SETTING (check_show_tabs, MOO_EDIT_PREFS_SHOW_TABS);
+    BIND_SETTING (check_show_spaces, MOO_EDIT_PREFS_SHOW_SPACES);
     BIND_SETTING (check_show_trailing_spaces, MOO_EDIT_PREFS_SHOW_TRAILING_SPACES);
     BIND_SETTING (draw_rigth_margin, MOO_EDIT_PREFS_DRAW_RIGHT_MARGIN);
     BIND_SETTING (spin_right_margin_offset, MOO_EDIT_PREFS_RIGHT_MARGIN_OFFSET);
+
+    moo_help_set_id (GTK_WIDGET (page), HELP_SECTION_PREFS_VIEW);
 }
 
 static void
@@ -284,7 +252,6 @@ GtkWidget *
 moo_edit_prefs_page_new_2 (MooEditor *editor)
 {
     return prefs_page_new (editor,
-                           /* Translators: this one is temporary, don't translate */
                            Q_("PreferencesPage|View"),
                            GTK_STOCK_EDIT,
                            page_view_init_ui,
@@ -307,6 +274,8 @@ page_file_init_ui (MooPrefsPage *page)
     BIND_SETTING (check_make_backups, MOO_EDIT_PREFS_MAKE_BACKUPS);
     BIND_SETTING (check_save_session, MOO_EDIT_PREFS_SAVE_SESSION);
     BIND_SETTING (check_open_dialog_follows_doc, MOO_EDIT_PREFS_DIALOGS_OPEN_FOLLOWS_DOC);
+
+    moo_help_set_id (GTK_WIDGET (page), HELP_SECTION_PREFS_FILE);
 }
 
 static void
@@ -329,7 +298,6 @@ GtkWidget *
 moo_edit_prefs_page_new_3 (MooEditor *editor)
 {
     return prefs_page_new (editor,
-                           /* Translators: this one is temporary, don't translate */
                            Q_("PreferencesPage|File"),
                            GTK_STOCK_EDIT,
                            page_file_init_ui,
@@ -344,6 +312,7 @@ page_langs_init_ui (MooPrefsPage *page)
     PrefsLangsXml *gxml;
     gxml = prefs_langs_xml_new_with_root (GTK_WIDGET (page));
     g_object_set_data (G_OBJECT (page), "moo-edit-prefs-page-xml", gxml);
+    moo_help_set_id (GTK_WIDGET (page), HELP_SECTION_PREFS_LANGS);
 }
 
 static void
@@ -351,40 +320,24 @@ page_langs_init (MooPrefsPage *page)
 {
     PrefsLangsXml *gxml = g_object_get_data (G_OBJECT (page), "moo-edit-prefs-page-xml");
     MooTreeHelper *helper;
-    const char *lang;
-
-    moo_help_set_id (GTK_WIDGET (page), HELP_SECTION_PREFS_LANGS_AND_FILTERS);
 
     lang_combo_init (gxml->lang_combo, page, gxml);
 
     helper = g_object_get_data (G_OBJECT (page), "moo-tree-helper");
     _moo_tree_helper_update_widgets (helper);
-
-    default_lang_combo_init (gxml->default_lang_combo, page);
-    lang = moo_prefs_get_string (moo_edit_setting (MOO_EDIT_PREFS_DEFAULT_LANG));
-    default_lang_combo_set_lang (gxml->default_lang_combo, lang);
 }
 
 static void
 page_langs_apply (MooPrefsPage *page)
 {
-    char *lang;
-    PrefsLangsXml *gxml = g_object_get_data (G_OBJECT (page), "moo-edit-prefs-page-xml");
-
     prefs_page_apply_lang_prefs (page);
-
-    lang = page_get_default_lang (gxml);
-    moo_prefs_set_string (moo_edit_setting (MOO_EDIT_PREFS_DEFAULT_LANG), lang);
-
-    g_free (lang);
 }
 
 GtkWidget *
 moo_edit_prefs_page_new_4 (MooEditor *editor)
 {
     return prefs_page_new (editor,
-                           /* Translators: this one is temporary, don't translate */
-                           Q_("PreferencesPage|Langs"),
+                           Q_("PreferencesPage|Languages"),
                            GTK_STOCK_EDIT,
                            page_langs_init_ui,
                            page_langs_init,
@@ -599,7 +552,7 @@ create_lang_model (void)
     mime = list_to_string (_moo_lang_mgr_get_mime_types (mgr, NULL), TRUE);
     gtk_tree_store_set (store, &iter,
                         COLUMN_ID, MOO_LANG_NONE,
-                        COLUMN_NAME, "None",
+                        COLUMN_NAME, Q_("Language|None"),
                         COLUMN_CONFIG, config,
                         COLUMN_MIMETYPES, mime,
                         COLUMN_EXTENSIONS, ext,
@@ -683,105 +636,6 @@ set_sensitive (G_GNUC_UNUSED GtkCellLayout *cell_layout,
                   NULL);
 }
 
-static void
-default_lang_combo_init (GtkComboBox  *combo,
-                         MooPrefsPage *page)
-{
-    GtkTreeModel *model;
-    GtkCellRenderer *cell;
-
-    fix_style (GTK_WIDGET (combo));
-
-    model = page_get_lang_model (page);
-    g_return_if_fail (model != NULL);
-
-    cell = gtk_cell_renderer_text_new ();
-    gtk_cell_layout_clear (GTK_CELL_LAYOUT (combo));
-    gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo), cell, TRUE);
-    gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (combo), cell,
-                                    "text", COLUMN_NAME, NULL);
-    gtk_cell_layout_set_cell_data_func (GTK_CELL_LAYOUT (combo), cell,
-                                        set_sensitive, NULL, NULL);
-
-    gtk_combo_box_set_model (combo, model);
-}
-
-
-static gboolean
-find_lang_by_id (GtkTreeModel *model,
-                 G_GNUC_UNUSED GtkTreePath *path,
-                 GtkTreeIter  *iter,
-                 gpointer      user_data)
-{
-    struct {
-        gboolean found;
-        GtkTreeIter iter;
-        const char *id;
-    } *data = user_data;
-
-    char *lang_id = NULL;
-
-    gtk_tree_model_get (model, iter, COLUMN_ID, &lang_id, -1);
-
-    if (_moo_str_equal(data->id, lang_id))
-    {
-        data->found = TRUE;
-        data->iter = *iter;
-        g_free (lang_id);
-        return TRUE;
-    }
-
-    g_free (lang_id);
-    return FALSE;
-}
-
-
-static void
-default_lang_combo_set_lang (GtkComboBox *combo,
-                             const char  *id)
-{
-    GtkTreeModel *model;
-
-    struct {
-        gboolean found;
-        GtkTreeIter iter;
-        const char *id;
-    } data;
-
-    g_return_if_fail (GTK_IS_COMBO_BOX (combo));
-
-    model = gtk_combo_box_get_model (combo);
-    data.found = FALSE;
-    data.id = id ? id : MOO_LANG_NONE;
-
-    gtk_tree_model_foreach (model,
-                            (GtkTreeModelForeachFunc) find_lang_by_id,
-                            &data);
-
-    g_return_if_fail (data.found);
-    gtk_combo_box_set_active_iter (combo, &data.iter);
-}
-
-
-static char *
-page_get_default_lang (PrefsLangsXml *gxml)
-{
-    GtkTreeModel *model;
-    GtkTreeIter iter;
-    char *lang = NULL;
-
-    if (!gtk_combo_box_get_active_iter (gxml->default_lang_combo, &iter))
-    {
-        g_critical ("%s: oops", G_STRLOC);
-        return NULL;
-    }
-
-    model = gtk_combo_box_get_model (gxml->default_lang_combo);
-    gtk_tree_model_get (model, &iter, COLUMN_ID, &lang, -1);
-
-    return lang;
-}
-
 
 /*********************************************************************/
 /* Per-lang settings
@@ -807,9 +661,9 @@ helper_update_widgets (PrefsLangsXml      *gxml,
                         -1);
     g_return_if_fail (id != NULL);
 
-    gtk_entry_set_text (gxml->extensions, ext ? ext : "");
-    gtk_entry_set_text (gxml->mimetypes, mime ? mime : "");
-    gtk_entry_set_text (gxml->config, conf ? conf : "");
+    gtk_entry_set_text (gxml->extensions, MOO_NZS (ext));
+    gtk_entry_set_text (gxml->mimetypes, MOO_NZS (mime));
+    gtk_entry_set_text (gxml->config, MOO_NZS (conf));
     gtk_widget_set_sensitive (GTK_WIDGET (gxml->mimetypes), lang != NULL);
     gtk_widget_set_sensitive (GTK_WIDGET (gxml->label_mimetypes), lang != NULL);
 
@@ -933,7 +787,7 @@ prefs_page_apply_lang_prefs (MooPrefsPage *page)
     mgr = moo_lang_mgr_default ();
     gtk_tree_model_foreach (model, (GtkTreeModelForeachFunc) apply_one_lang, mgr);
     _moo_lang_mgr_save_config (mgr);
-    _moo_edit_update_lang_config ();
+    _moo_edit_queue_recheck_config_all ();
 }
 
 
@@ -942,6 +796,8 @@ prefs_page_apply_lang_prefs (MooPrefsPage *page)
  */
 
 enum {
+    FILTER_COLUMN_INVALID,
+    FILTER_COLUMN_INVALID_TOOLTIP,
     FILTER_COLUMN_FILTER,
     FILTER_COLUMN_CONFIG,
     FILTER_NUM_COLUMNS
@@ -966,6 +822,65 @@ filter_store_get_modified (gpointer store)
            _moo_tree_helper_get_modified (g_object_get_data (store, "tree-helper"));
 }
 
+static gboolean
+check_filter (const char  *string,
+              char       **message)
+{
+    gboolean result;
+    MooEditFilter *filter;
+    GError *error = NULL;
+
+    if (message)
+        *message = NULL;
+
+    if (!string || !string[0])
+        return TRUE;
+
+    filter = _moo_edit_filter_new_full (string, MOO_EDIT_FILTER_CONFIG, &error);
+    g_return_val_if_fail (filter != NULL, FALSE);
+
+    result = _moo_edit_filter_valid (filter);
+
+    if (error && message)
+        *message = g_strdup (error->message);
+
+    if (error)
+        g_error_free (error);
+
+    _moo_edit_filter_free (filter);
+    return result;
+}
+
+static void
+check_filter_row (GtkListStore *store,
+                  GtkTreeIter  *iter)
+{
+    gboolean invalid;
+    char *filter = NULL;
+    char *message = NULL;
+
+    gtk_tree_model_get (GTK_TREE_MODEL (store), iter,
+                        FILTER_COLUMN_FILTER, &filter,
+                        -1);
+    g_return_if_fail (filter != NULL);
+
+    invalid = !check_filter (filter, &message);
+
+    if (invalid)
+        gtk_list_store_set (store, iter,
+                            FILTER_COLUMN_INVALID, TRUE,
+                            FILTER_COLUMN_INVALID_TOOLTIP, message,
+                            -1);
+    else
+        gtk_list_store_set (store, iter,
+                            FILTER_COLUMN_INVALID, FALSE,
+                            FILTER_COLUMN_INVALID_TOOLTIP, NULL,
+                            -1);
+
+    g_free (message);
+    g_free (filter);
+}
+
 static void
 populate_filter_settings_store (GtkListStore *store)
 {
@@ -978,15 +893,27 @@ populate_filter_settings_store (GtkListStore *store)
     while (l)
     {
         GtkTreeIter iter;
+        const char *filter;
+        const char *config;
+        gboolean valid;
+        char *message = NULL;
 
         g_return_if_fail (l->next != NULL);
 
+        filter = l->data;
+        config = l->next->data;
+        valid = check_filter (filter, &message);
+
         gtk_list_store_append (store, &iter);
         gtk_list_store_set (store, &iter,
-                            FILTER_COLUMN_FILTER, l->data,
-                            FILTER_COLUMN_CONFIG, l->next->data,
+                            FILTER_COLUMN_INVALID, !valid,
+                            FILTER_COLUMN_INVALID_TOOLTIP, message,
+                            FILTER_COLUMN_FILTER, filter,
+                            FILTER_COLUMN_CONFIG, config,
                             -1);
 
+        g_free (message);
+
         l = l->next->next;
     }
 
@@ -1021,10 +948,11 @@ filter_cell_edited (GtkCellRendererText *cell,
 
     gtk_tree_model_get (GTK_TREE_MODEL (store), &iter, column, &old_text, -1);
 
-    if (!_moo_str_equal (old_text, text))
+    if (!moo_str_equal (old_text, text))
     {
         gtk_list_store_set (store, &iter, column, text, -1);
         filter_store_set_modified (store, TRUE);
+        check_filter_row (store, &iter);
     }
 
     g_free (old_text);
@@ -1033,18 +961,43 @@ filter_cell_edited (GtkCellRendererText *cell,
 
 
 static void
-create_filter_cell (GtkTreeView  *treeview,
-                    GtkListStore *store,
-                    const char   *title,
-                    int           column_id)
+filter_icon_data_func (G_GNUC_UNUSED GtkTreeViewColumn *column,
+                       GtkCellRenderer *cell,
+                       GtkTreeModel    *model,
+                       GtkTreeIter     *iter)
+{
+    gboolean invalid;
+    gtk_tree_model_get (model, iter, FILTER_COLUMN_INVALID, &invalid, -1);
+    g_object_set (cell, "visible", invalid, NULL);
+}
+
+static void
+create_filter_column (GtkTreeView  *treeview,
+                      GtkListStore *store,
+                      const char   *title,
+                      int           column_id)
 {
     GtkTreeViewColumn *column;
     GtkCellRenderer *cell;
 
-    cell = gtk_cell_renderer_text_new ();
-    column = gtk_tree_view_column_new_with_attributes (title, cell, "text", column_id, NULL);
+    column = gtk_tree_view_column_new ();
+    gtk_tree_view_column_set_title (column, title);
     gtk_tree_view_append_column (treeview, column);
 
+    if (column_id == FILTER_COLUMN_FILTER)
+    {
+        cell = gtk_cell_renderer_pixbuf_new ();
+        g_object_set (cell, "stock-id", GTK_STOCK_DIALOG_ERROR, NULL);
+        gtk_tree_view_column_pack_start (column, cell, FALSE);
+        gtk_tree_view_column_set_cell_data_func (column, cell,
+                                                 (GtkTreeCellDataFunc) filter_icon_data_func,
+                                                 NULL, NULL);
+    }
+
+    cell = gtk_cell_renderer_text_new ();
+    gtk_tree_view_column_pack_start (column, cell, TRUE);
+    gtk_tree_view_column_set_attributes (column, cell, "text", column_id, NULL);
+
     g_object_set (cell, "editable", TRUE, NULL);
     g_object_set_data (G_OBJECT (cell), "filter-store-column-id", GINT_TO_POINTER (column_id));
     g_signal_connect (cell, "edited", G_CALLBACK (filter_cell_edited), store);
@@ -1057,12 +1010,16 @@ filter_treeview_init (PrefsFiltersXml *gxml)
     GtkListStore *store;
     MooTreeHelper *helper;
 
-    store = gtk_list_store_new (FILTER_NUM_COLUMNS, G_TYPE_STRING, G_TYPE_STRING);
+    store = gtk_list_store_new (FILTER_NUM_COLUMNS, G_TYPE_BOOLEAN, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
     populate_filter_settings_store (store);
     gtk_tree_view_set_model (gxml->filter_treeview, GTK_TREE_MODEL (store));
 
-    create_filter_cell (gxml->filter_treeview, store, "Filter", FILTER_COLUMN_FILTER);
-    create_filter_cell (gxml->filter_treeview, store, "Options", FILTER_COLUMN_CONFIG);
+    /* Column label on File Filters prefs page */
+    create_filter_column (gxml->filter_treeview, store, C_("filter-prefs-column", "Filter"), FILTER_COLUMN_FILTER);
+    /* Column label on File Filters prefs page */
+    create_filter_column (gxml->filter_treeview, store, C_("filter-prefs-column", "Options"), FILTER_COLUMN_CONFIG);
+
+    gtk_tree_view_set_tooltip_column (gxml->filter_treeview, FILTER_COLUMN_INVALID_TOOLTIP);
 
     helper = _moo_tree_helper_new (GTK_WIDGET (gxml->filter_treeview),
                                    GTK_WIDGET (gxml->new_filter_setting),
