@@ -1,7 +1,7 @@
 /*
  *   moolangmgr.c
  *
- *   Copyright (C) 2004-2009 by Yevgen Muntyan <muntyan@tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
  *   This file is part of medit.  medit is free software; you can
  *   redistribute it and/or modify it under the terms of the
@@ -34,7 +34,7 @@
 #define ELEMENT_MIME_TYPES      "mime-types"
 #define ELEMENT_CONFIG          "config"
 #define PROP_LANG_ID            "id"
-#define SCHEME_DEFAULT          "kate"
+#define SCHEME_DEFAULT          "medit"
 
 
 typedef struct {
@@ -66,7 +66,6 @@ static void
 moo_lang_mgr_init (MooLangMgr *mgr)
 {
     char **dirs;
-    guint n_dirs;
 
     mgr->schemes = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_object_unref);
 
@@ -76,7 +75,7 @@ moo_lang_mgr_init (MooLangMgr *mgr)
                                          g_free, g_free);
 
     mgr->lang_mgr = gtk_source_language_manager_new ();
-    dirs = moo_get_data_subdirs (LANGUAGE_DIR, MOO_DATA_SHARE, &n_dirs);
+    dirs = moo_get_data_subdirs (LANGUAGE_DIR);
     g_object_set (mgr->lang_mgr, "search-path", dirs, NULL);
     mgr->style_mgr = gtk_source_style_scheme_manager_new ();
     gtk_source_style_scheme_manager_set_search_path (mgr->style_mgr, dirs);
@@ -127,7 +126,7 @@ moo_lang_mgr_class_init (MooLangMgrClass *klass)
 MooLangMgr *
 moo_lang_mgr_new (void)
 {
-    return g_object_new (MOO_TYPE_LANG_MGR, NULL);
+    return MOO_LANG_MGR (g_object_new (MOO_TYPE_LANG_MGR, (const char*) NULL));
 }
 
 MooLangMgr *
@@ -212,7 +211,7 @@ moo_lang_mgr_get_lang (MooLangMgr *mgr,
 {
     MooLang *lang;
 
-    moo_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
+    g_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
 
     if (!name)
         return NULL;
@@ -220,7 +219,7 @@ moo_lang_mgr_get_lang (MooLangMgr *mgr,
     lang = _moo_lang_mgr_find_lang (mgr, name);
 
     if (!lang)
-        moo_message ("could not find language '%s'", name);
+        g_message ("could not find language '%s'", name);
 
     return lang;
 }
@@ -232,8 +231,8 @@ _moo_lang_mgr_find_lang (MooLangMgr *mgr,
     char *id;
     LangInfo *info;
 
-    moo_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
-    moo_return_val_if_fail (name != NULL, NULL);
+    g_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
+    g_return_val_if_fail (name != NULL, NULL);
 
     if (!strcmp (name, MOO_LANG_NONE))
         return NULL;
@@ -259,19 +258,16 @@ get_lang_by_extension (MooLangMgr *mgr,
                        const char *filename)
 {
     MooLang *lang = NULL;
-    char *basename, *utf8_basename;
+    char *basename;
     GSList *langs, *l;
     gboolean found = FALSE;
 
-    moo_return_val_if_fail (filename != NULL, NULL);
+    g_return_val_if_fail (filename != NULL, NULL);
 
     read_langs (mgr);
 
-    /* TODO: is this right? */
     basename = g_path_get_basename (filename);
-    moo_return_val_if_fail (basename != NULL, NULL);
-    utf8_basename = g_filename_display_name (basename);
-    moo_return_val_if_fail (utf8_basename != NULL, NULL);
+    g_return_val_if_fail (basename != NULL, NULL);
 
     langs = moo_lang_mgr_get_available_langs (mgr);
 
@@ -284,7 +280,7 @@ get_lang_by_extension (MooLangMgr *mgr,
 
         for (g = globs; !found && g != NULL; g = g->next)
         {
-            if (_moo_glob_match_simple ((char*) g->data, utf8_basename))
+            if (_moo_glob_match_simple ((char*) g->data, basename))
             {
                 found = TRUE;
                 break;
@@ -299,7 +295,6 @@ get_lang_by_extension (MooLangMgr *mgr,
 
     g_slist_foreach (langs, (GFunc) g_object_unref, NULL);
     g_slist_free (langs);
-    g_free (utf8_basename);
     g_free (basename);
     return lang;
 }
@@ -310,7 +305,7 @@ lang_mgr_get_lang_for_bak_filename (MooLangMgr *mgr,
                                     const char *filename)
 {
     MooLang *lang = NULL;
-    char *utf8_name, *utf8_base = NULL;
+    char *base = NULL;
     int len;
     guint i;
 
@@ -318,32 +313,23 @@ lang_mgr_get_lang_for_bak_filename (MooLangMgr *mgr,
 
     read_langs (mgr);
 
-    utf8_name = g_filename_display_name (filename);
-    len = strlen (utf8_name);
+    len = strlen (filename);
 
     for (i = 0; i < G_N_ELEMENTS (bak_globs); ++i)
     {
         int ext_len = strlen (bak_globs[i]) - 1;
 
-        if (len > ext_len && _moo_glob_match_simple (bak_globs[i], utf8_name))
+        if (len > ext_len && _moo_glob_match_simple (bak_globs[i], filename))
         {
-            utf8_base = g_strndup (utf8_name, len - ext_len);
+            base = g_strndup (filename, len - ext_len);
             break;
         }
     }
 
-    if (utf8_base)
-    {
-        char *base = g_filename_from_utf8 (utf8_base, -1, NULL, NULL, NULL);
+    if (base)
+        lang = get_lang_for_filename (mgr, base);
 
-        if (base)
-            lang = get_lang_for_filename (mgr, base);
-
-        g_free (base);
-        g_free (utf8_base);
-    }
-
-    g_free (utf8_name);
+    g_free (base);
     return lang;
 }
 
@@ -353,7 +339,7 @@ filename_blacklisted (MooLangMgr *mgr,
                       const char *filename)
 {
     /* XXX bak files */
-    char *basename, *utf8_basename;
+    char *basename;
     gboolean result = FALSE;
     LangInfo *info;
     GSList *l;
@@ -361,18 +347,15 @@ filename_blacklisted (MooLangMgr *mgr,
     read_langs (mgr);
 
     basename = g_path_get_basename (filename);
-    moo_return_val_if_fail (basename != NULL, FALSE);
-    utf8_basename = g_filename_display_name (basename);
-    moo_return_val_if_fail (utf8_basename != NULL, FALSE);
+    g_return_val_if_fail (basename != NULL, FALSE);
 
     info = get_lang_info (mgr, MOO_LANG_NONE, FALSE);
 
     if (info)
         for (l = info->globs; !result && l != NULL; l = l->next)
-            if (_moo_glob_match_simple ((char*) l->data, utf8_basename))
+            if (_moo_glob_match_simple ((char*) l->data, basename))
                 result = TRUE;
 
-    g_free (utf8_basename);
     g_free (basename);
     return result;
 }
@@ -388,28 +371,31 @@ file_blacklisted (MooLangMgr *mgr,
 
 MooLang *
 moo_lang_mgr_get_lang_for_file (MooLangMgr *mgr,
-                                const char *filename)
+                                GFile      *file)
 {
     MooLang *lang = NULL;
     const char *mime_type;
+    char *filename;
 
-    moo_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
-    moo_return_val_if_fail (filename != NULL, NULL);
+    g_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
+    g_return_val_if_fail (G_IS_FILE (file), NULL);
 
     read_langs (mgr);
 
+    filename = g_file_get_parse_name (file);
+
     if (file_blacklisted (mgr, filename))
-        return NULL;
+        goto out;
 
     lang = get_lang_by_extension (mgr, filename);
 
     if (lang)
-        return lang;
+        goto out;
 
     lang = lang_mgr_get_lang_for_bak_filename (mgr, filename);
 
     if (lang)
-        return lang;
+        goto out;
 
     mime_type = moo_get_mime_type_for_file (filename, NULL);
 
@@ -417,9 +403,11 @@ moo_lang_mgr_get_lang_for_file (MooLangMgr *mgr,
         lang = get_lang_for_mime_type (mgr, mime_type);
 
     if (lang)
-        return lang;
+        goto out;
 
-    return NULL;
+out:
+    g_free (filename);
+    return lang;
 }
 
 
@@ -430,8 +418,8 @@ get_lang_for_filename (MooLangMgr *mgr,
     MooLang *lang = NULL;
     const char *mime_type;
 
-    moo_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
-    moo_return_val_if_fail (filename != NULL, NULL);
+    g_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
+    g_return_val_if_fail (filename != NULL, NULL);
 
     read_langs (mgr);
 
@@ -476,8 +464,8 @@ get_lang_for_mime_type (MooLangMgr *mgr,
     MooLang *lang = NULL;
     gboolean found = FALSE;
 
-    moo_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
-    moo_return_val_if_fail (mime != NULL, NULL);
+    g_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
+    g_return_val_if_fail (mime != NULL, NULL);
 
     langs = moo_lang_mgr_get_available_langs (mgr);
 
@@ -525,9 +513,7 @@ read_langs (MooLangMgr *mgr)
 
     mgr->got_langs = TRUE;
 
-#ifdef MOO_USE_XML
     ids = gtk_source_language_manager_get_language_ids (mgr->lang_mgr);
-#endif
 
     get_lang_info (mgr, MOO_LANG_NONE, TRUE);
 
@@ -587,7 +573,7 @@ load_lang_node (MooLangMgr    *mgr,
     MooMarkupNode *node;
 
     lang_id = moo_markup_get_prop (lang_node, PROP_LANG_ID);
-    moo_return_if_fail (lang_id != NULL);
+    g_return_if_fail (lang_id != NULL);
 
     for (node = lang_node->children; node != NULL; node = node->next)
     {
@@ -611,7 +597,7 @@ load_lang_node (MooLangMgr    *mgr,
         {
             /* 'extensions' is leftover from previous versions */
             if (strcmp (node->name, "extensions") != 0)
-                g_warning ("%s: ignoring node '%s'", G_STRLOC, node->name);
+                g_warning ("ignoring node '%s'", node->name);
         }
     }
 }
@@ -627,17 +613,16 @@ set_default_config (MooLangMgr *mgr)
 static void
 load_config (MooLangMgr *mgr)
 {
-    MooMarkupDoc *xml;
+    MooMarkupNode *xml;
     MooMarkupNode *root, *node;
 
     set_default_config (mgr);
     mgr->modified = FALSE;
 
     xml = moo_prefs_get_markup (MOO_PREFS_RC);
-    moo_return_if_fail (xml != NULL);
+    g_return_if_fail (xml != NULL);
 
-    root = moo_markup_get_element (MOO_MARKUP_NODE (xml),
-                                   ELEMENT_LANG_CONFIG);
+    root = moo_markup_get_element (xml, ELEMENT_LANG_CONFIG);
 
     if (!root)
         return;
@@ -649,7 +634,7 @@ load_config (MooLangMgr *mgr)
 
         if (strcmp (node->name, ELEMENT_LANG))
         {
-            g_warning ("%s: invalid '%s' element", G_STRLOC, node->name);
+            g_warning ("invalid '%s' element", node->name);
             continue;
         }
 
@@ -663,7 +648,7 @@ load_config (MooLangMgr *mgr)
 MooTextStyleScheme *
 moo_lang_mgr_get_active_scheme (MooLangMgr *mgr)
 {
-    moo_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
+    g_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
 
     read_schemes (mgr);
 
@@ -677,7 +662,7 @@ _moo_lang_mgr_set_active_scheme (MooLangMgr *mgr,
 {
     MooTextStyleScheme *scheme = NULL;
 
-    moo_return_if_fail (MOO_IS_LANG_MGR (mgr));
+    g_return_if_fail (MOO_IS_LANG_MGR (mgr));
 
     read_schemes (mgr);
 
@@ -687,7 +672,7 @@ _moo_lang_mgr_set_active_scheme (MooLangMgr *mgr,
         scheme = g_hash_table_lookup (mgr->schemes, SCHEME_DEFAULT);
 
     if (!scheme)
-        moo_message ("could not find style scheme '%s'", name ? name : "<NULL>");
+        g_message ("could not find style scheme '%s'", name ? name : "<NULL>");
     else
         mgr->active_scheme = scheme;
 }
@@ -706,7 +691,7 @@ moo_lang_mgr_list_schemes (MooLangMgr *mgr)
 {
     GSList *list = NULL;
 
-    moo_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
+    g_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
 
     read_schemes (mgr);
     g_hash_table_foreach (mgr->schemes, (GHFunc) prepend_scheme, &list);
@@ -735,7 +720,7 @@ read_schemes (MooLangMgr *mgr)
 
         if (!scheme)
         {
-            g_critical ("%s: oops", G_STRLOC);
+            g_critical ("oops");
         }
         else
         {
@@ -755,13 +740,11 @@ moo_lang_mgr_get_sections (MooLangMgr *mgr)
     GSList *sections = NULL;
     const char * const *ids = NULL;
 
-    moo_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
+    g_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
 
     read_langs (mgr);
 
-#ifdef MOO_USE_XML
     ids = gtk_source_language_manager_get_language_ids (mgr->lang_mgr);
-#endif
 
     while (ids && *ids)
     {
@@ -789,13 +772,11 @@ moo_lang_mgr_get_available_langs (MooLangMgr *mgr)
     GSList *list = NULL;
     const char *const *ids = NULL;
 
-    moo_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
+    g_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
 
     read_langs (mgr);
 
-#ifdef MOO_USE_XML
     ids = gtk_source_language_manager_get_language_ids (mgr->lang_mgr);
-#endif
 
     while (ids && *ids)
     {
@@ -817,13 +798,13 @@ _moo_lang_mgr_get_globs (MooLangMgr *mgr,
     char *id;
     LangInfo *info;
 
-    moo_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
+    g_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
 
     read_langs (mgr);
 
     id = _moo_lang_id_from_name (lang_id);
     info = get_lang_info (mgr, id, FALSE);
-    moo_return_val_if_fail (info != NULL, NULL);
+    g_return_val_if_fail (info != NULL, NULL);
 
     g_free (id);
     return string_list_copy (info->globs);
@@ -837,13 +818,13 @@ _moo_lang_mgr_get_mime_types (MooLangMgr *mgr,
     char *id;
     LangInfo *info;
 
-    moo_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
+    g_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
 
     read_langs (mgr);
 
     id = _moo_lang_id_from_name (lang_id);
     info = get_lang_info (mgr, id, FALSE);
-    moo_return_val_if_fail (info != NULL, NULL);
+    g_return_val_if_fail (info != NULL, NULL);
 
     g_free (id);
     return string_list_copy (info->mime_types);
@@ -885,7 +866,7 @@ set_globs_or_mime_types (MooLangMgr *mgr,
     id = _moo_lang_id_from_name (lang_id);
     info = get_lang_info (mgr, id, FALSE);
 
-    moo_return_if_fail (info != NULL);
+    g_return_if_fail (info != NULL);
 
     new = _moo_lang_parse_string_list (string);
     old = globs ? _moo_lang_mgr_get_globs (mgr, id) :
@@ -935,7 +916,7 @@ _moo_lang_mgr_set_globs (MooLangMgr *mgr,
                          const char *lang_id,
                          const char *globs)
 {
-    moo_return_if_fail (MOO_IS_LANG_MGR (mgr));
+    g_return_if_fail (MOO_IS_LANG_MGR (mgr));
     set_globs_or_mime_types (mgr, lang_id, globs, TRUE);
 }
 
@@ -945,7 +926,7 @@ _moo_lang_mgr_set_mime_types (MooLangMgr *mgr,
                               const char *lang_id,
                               const char *mime)
 {
-    moo_return_if_fail (MOO_IS_LANG_MGR (mgr));
+    g_return_if_fail (MOO_IS_LANG_MGR (mgr));
     set_globs_or_mime_types (mgr, lang_id, mime, FALSE);
 }
 
@@ -957,7 +938,7 @@ _moo_lang_mgr_get_config (MooLangMgr *mgr,
     char *id;
     const char *config;
 
-    moo_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
+    g_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
 
     read_langs (mgr);
 
@@ -978,7 +959,7 @@ _moo_lang_mgr_set_config (MooLangMgr *mgr,
     char *norm = NULL;
     const char *old;
 
-    moo_return_if_fail (MOO_IS_LANG_MGR (mgr));
+    g_return_if_fail (MOO_IS_LANG_MGR (mgr));
 
     read_langs (mgr);
 
@@ -996,7 +977,7 @@ _moo_lang_mgr_set_config (MooLangMgr *mgr,
     id = _moo_lang_id_from_name (lang_id);
     old = g_hash_table_lookup (mgr->config, id);
 
-    if (!_moo_str_equal (old, norm))
+    if (!moo_str_equal (old, norm))
         mgr->modified = TRUE;
 
     g_hash_table_insert (mgr->config, id, norm);
@@ -1010,8 +991,8 @@ _moo_lang_mgr_update_config (MooLangMgr     *mgr,
 {
     const char *lang_config;
 
-    moo_return_if_fail (MOO_IS_LANG_MGR (mgr));
-    moo_return_if_fail (MOO_IS_EDIT_CONFIG (config));
+    g_return_if_fail (MOO_IS_LANG_MGR (mgr));
+    g_return_if_fail (MOO_IS_EDIT_CONFIG (config));
 
     read_langs (mgr);
 
@@ -1055,7 +1036,7 @@ save_one (const char *lang_id,
 
     struct {
         MooLangMgr *mgr;
-        MooMarkupDoc *xml;
+        MooMarkupNode *xml;
         MooMarkupNode *root;
     } *data;
 
@@ -1066,8 +1047,7 @@ save_one (const char *lang_id,
         return;
 
     if (!data->root)
-        data->root = moo_markup_create_element (MOO_MARKUP_NODE (data->xml),
-                                                ELEMENT_LANG_CONFIG);
+        data->root = moo_markup_create_element (data->xml, ELEMENT_LANG_CONFIG);
 
     lang_node = moo_markup_create_element (data->root, ELEMENT_LANG);
     moo_markup_set_prop (lang_node, PROP_LANG_ID, lang_id);
@@ -1075,14 +1055,14 @@ save_one (const char *lang_id,
     if (info->globs_modified)
     {
         char *string = list_to_string (info->globs);
-        moo_markup_create_text_element (lang_node, ELEMENT_EXTENSIONS, string ? string : "");
+        moo_markup_create_text_element (lang_node, ELEMENT_EXTENSIONS, string);
         g_free (string);
     }
 
     if (info->mime_types_modified)
     {
         char *string = list_to_string (info->mime_types);
-        moo_markup_create_text_element (lang_node, ELEMENT_EXTENSIONS, string ? string : "");
+        moo_markup_create_text_element (lang_node, ELEMENT_EXTENSIONS, string);
         g_free (string);
     }
 
@@ -1093,25 +1073,25 @@ save_one (const char *lang_id,
 void
 _moo_lang_mgr_save_config (MooLangMgr *mgr)
 {
-    MooMarkupDoc *xml;
+    MooMarkupNode *xml;
     MooMarkupNode *root;
 
     struct {
         MooLangMgr *mgr;
-        MooMarkupDoc *xml;
+        MooMarkupNode *xml;
         MooMarkupNode *root;
     } data;
 
-    moo_return_if_fail (MOO_IS_LANG_MGR (mgr));
+    g_return_if_fail (MOO_IS_LANG_MGR (mgr));
 
     if (!mgr->modified)
         return;
 
     xml = moo_prefs_get_markup (MOO_PREFS_RC);
-    moo_return_if_fail (xml != NULL);
+    g_return_if_fail (xml != NULL);
 
     mgr->modified = FALSE;
-    root = moo_markup_get_element (MOO_MARKUP_NODE (xml), ELEMENT_LANG_CONFIG);
+    root = moo_markup_get_element (xml, ELEMENT_LANG_CONFIG);
 
     if (root)
         moo_markup_delete_node (root);
