@@ -1,7 +1,7 @@
 /*
  *   moofileentry.c
  *
- *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
  *   This file is part of medit.  medit is free software; you can
  *   redistribute it and/or modify it under the terms of the
@@ -25,6 +25,7 @@
 #include "mooutils/mooutils-misc.h"
 #include "mooutils/mooutils-debug.h"
 #include "mooutils/mooaccel.h"
+#include "mooutils/moocompat.h"
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 
@@ -169,7 +170,7 @@ _moo_file_entry_completion_class_init (MooFileEntryCompletionClass *klass)
                                              "enable-completion",
                                              "enable-completion",
                                              TRUE,
-                                             G_PARAM_CONSTRUCT | G_PARAM_READWRITE));
+                                             (GParamFlags) (G_PARAM_CONSTRUCT | G_PARAM_READWRITE)));
 
     g_object_class_install_property (gobject_class,
                                      PROP_DIRECTORIES_ONLY,
@@ -177,7 +178,7 @@ _moo_file_entry_completion_class_init (MooFileEntryCompletionClass *klass)
                                              "directories-only",
                                              "directories-only",
                                              FALSE,
-                                             G_PARAM_CONSTRUCT | G_PARAM_READWRITE));
+                                             (GParamFlags) (G_PARAM_CONSTRUCT | G_PARAM_READWRITE)));
 
     g_object_class_install_property (gobject_class,
                                      PROP_SHOW_HIDDEN,
@@ -185,7 +186,7 @@ _moo_file_entry_completion_class_init (MooFileEntryCompletionClass *klass)
                                              "show-hidden",
                                              "show-hidden",
                                              TRUE,
-                                             G_PARAM_CONSTRUCT | G_PARAM_READWRITE));
+                                             (GParamFlags) (G_PARAM_CONSTRUCT | G_PARAM_READWRITE)));
 
     g_object_class_install_property (gobject_class,
                                      PROP_ENTRY,
@@ -193,7 +194,7 @@ _moo_file_entry_completion_class_init (MooFileEntryCompletionClass *klass)
                                              "entry",
                                              "entry",
                                              GTK_TYPE_ENTRY,
-                                             G_PARAM_CONSTRUCT | G_PARAM_READWRITE));
+                                             (GParamFlags) (G_PARAM_CONSTRUCT | G_PARAM_READWRITE)));
 
     g_object_class_install_property (gobject_class,
                                      PROP_CASE_SENSITIVE,
@@ -201,7 +202,7 @@ _moo_file_entry_completion_class_init (MooFileEntryCompletionClass *klass)
                                              "case-sensitive",
                                              "case-sensitive",
                                              CASE_SENSITIVE_DEFAULT,
-                                             G_PARAM_CONSTRUCT | G_PARAM_READWRITE));
+                                             (GParamFlags) (G_PARAM_CONSTRUCT | G_PARAM_READWRITE)));
 
     g_object_class_install_property (gobject_class,
                                      PROP_CURRENT_DIR,
@@ -209,7 +210,7 @@ _moo_file_entry_completion_class_init (MooFileEntryCompletionClass *klass)
                                              "current-dir",
                                              "current-dir",
                                              NULL,
-                                             G_PARAM_CONSTRUCT | G_PARAM_READWRITE));
+                                             (GParamFlags) (G_PARAM_CONSTRUCT | G_PARAM_READWRITE)));
 
     g_object_class_install_property (gobject_class,
                                      PROP_FILE_SYSTEM,
@@ -217,7 +218,7 @@ _moo_file_entry_completion_class_init (MooFileEntryCompletionClass *klass)
                                              "file-system",
                                              "file-system",
                                              MOO_TYPE_FILE_SYSTEM,
-                                             G_PARAM_CONSTRUCT | G_PARAM_READWRITE));
+                                             (GParamFlags) (G_PARAM_CONSTRUCT | G_PARAM_READWRITE)));
 
     signals[FINISHED] =
             _moo_signal_new_cb ("finished",
@@ -541,8 +542,8 @@ completion_parse_text (MooFileEntryCompletion *cmpl,
                                       path, &dirname, &display_dirname,
                                       &display_basename, &error))
     {
-        _moo_message ("%s: could not parse path '%s'", G_STRLOC, path);
-        _moo_message ("%s: %s", G_STRLOC, error->message);
+        _moo_message ("could not parse path '%s'", path);
+        _moo_message ("%s", moo_error_message (error));
         goto out;
     }
 
@@ -555,7 +556,7 @@ completion_parse_text (MooFileEntryCompletion *cmpl,
     /* this was needed in the widget */
     if (text_len <= cmpl->priv->display_basename_len)
     {
-        g_warning ("%s: something wrong", G_STRLOC);
+        g_warning ("something is wrong");
         goto out;
     }
 #endif
@@ -570,8 +571,8 @@ completion_parse_text (MooFileEntryCompletion *cmpl,
                                               &error);
         if (!folder)
         {
-            _moo_message ("%s: could not get folder '%s'", G_STRLOC, dirname);
-            _moo_message ("%s: %s", G_STRLOC, error->message);
+            _moo_message ("could not get folder '%s'", dirname);
+            _moo_message ("%s", moo_error_message (error));
             g_error_free (error);
             goto out;
         }
@@ -645,7 +646,7 @@ completion_tab_key (MooFileEntryCompletion *cmpl)
 
         if (!prefix)
         {
-            g_critical ("%s: oops", G_STRLOC);
+            g_critical ("oops");
         }
         else if (prefix->len > cmpl->priv->display_basename_len)
         {
@@ -712,7 +713,7 @@ completion_popup_tab_key_press (MooFileEntryCompletion *cmpl)
 
         if (!prefix)
         {
-            g_critical ("%s: oops", G_STRLOC);
+            g_critical ("oops");
             return FALSE;
         }
 
@@ -788,7 +789,7 @@ completion_popup (MooFileEntryCompletion *cmpl)
 
     gtk_widget_realize (cmpl->priv->popup);
 
-    GTK_WIDGET_SET_FLAGS (GTK_WIDGET (cmpl->priv->treeview), GTK_CAN_FOCUS);
+    GTK_WIDGET_SET_CAN_FOCUS (cmpl->priv->treeview);
 
     if (GTK_WINDOW (window)->group)
         gtk_window_group_add_window (GTK_WINDOW (window)->group,
@@ -1170,7 +1171,7 @@ folder_contents_changed (MooFileEntryCompletion *cmpl)
 {
     if (!cmpl->priv->resize_popup_idle)
         cmpl->priv->resize_popup_idle =
-            moo_idle_add ((GSourceFunc) resize_popup_idle, cmpl);
+            gdk_threads_add_idle ((GSourceFunc) resize_popup_idle, cmpl);
 }
 
 static void
@@ -1634,7 +1635,7 @@ _moo_file_entry_class_init (MooFileEntryClass *klass)
 static void
 _moo_file_entry_init (G_GNUC_UNUSED MooFileEntry *entry)
 {
-    entry->completion = g_object_new (MOO_TYPE_FILE_ENTRY_COMPLETION, NULL);
+    entry->completion = MOO_FILE_ENTRY_COMPLETION (g_object_new (MOO_TYPE_FILE_ENTRY_COMPLETION, (const char*) NULL));
     entry->completion->priv->managed = TRUE;
     _moo_file_entry_completion_set_entry (entry->completion, GTK_ENTRY (entry));
 }
@@ -1692,10 +1693,10 @@ moo_file_entry_get_property (GObject        *object,
 }
 
 
-GtkWidget*
+GtkWidget *
 _moo_file_entry_new (void)
 {
-    return g_object_new (MOO_TYPE_FILE_ENTRY, NULL);
+    return GTK_WIDGET (g_object_new (MOO_TYPE_FILE_ENTRY, (const char*) NULL));
 }
 
 
