@@ -1,7 +1,7 @@
 /*
  *   mooaccel.c
  *
- *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
  *   This file is part of medit.  medit is free software; you can
  *   redistribute it and/or modify it under the terms of the
@@ -15,13 +15,16 @@
 
 #include "mooutils/mooaccel.h"
 #include "mooutils/mooprefs.h"
+#include "mooutils/mooutils-tests.h"
+#include "mooutils/moocompat.h"
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 #include <string.h>
+#include <locale.h>
 
 #define MOO_ACCEL_PREFS_KEY "Shortcuts"
 
-#if GTK_CHECK_VERSION(2,10,0) && defined(GDK_WINDOWING_QUARTZ)
+#if defined(GDK_WINDOWING_QUARTZ)
 #define COMMAND_MASK GDK_META_MASK
 #else
 #define COMMAND_MASK GDK_CONTROL_MASK
@@ -218,7 +221,7 @@ set_accel (const char *accel_path,
            gboolean    set_gtk)
 {
     guint accel_key = 0;
-    GdkModifierType accel_mods = 0;
+    GdkModifierType accel_mods = (GdkModifierType) 0;
     const char *old_accel;
 
     g_return_if_fail (accel_path != NULL && accel != NULL);
@@ -277,7 +280,7 @@ get_accel (const char *accel_path)
 {
     g_return_val_if_fail (accel_path != NULL, NULL);
     init_accel_map ();
-    return g_hash_table_lookup (moo_accel_map, accel_path);
+    return (const char*) g_hash_table_lookup (moo_accel_map, accel_path);
 }
 
 
@@ -286,7 +289,7 @@ get_default_accel (const char *accel_path)
 {
     g_return_val_if_fail (accel_path != NULL, NULL);
     init_accel_map ();
-    return g_hash_table_lookup (moo_default_accel_map, accel_path);
+    return (const char*) g_hash_table_lookup (moo_default_accel_map, accel_path);
 }
 
 
@@ -332,7 +335,7 @@ _moo_accel_register (const char *accel_path,
     if (default_accel[0])
     {
         guint accel_key = 0;
-        GdkModifierType accel_mods = 0;
+        GdkModifierType accel_mods = (GdkModifierType) 0;
 
         my_gtk_accelerator_parse (default_accel, &accel_key, &accel_mods);
 
@@ -395,7 +398,7 @@ char *
 _moo_get_accel_label (const char *accel)
 {
     guint key = 0;
-    GdkModifierType mods = 0;
+    GdkModifierType mods = (GdkModifierType) 0;
 
     g_return_val_if_fail (accel != NULL, g_strdup (""));
 
@@ -423,7 +426,7 @@ moo_accel_translate_event (GtkWidget       *widget,
     if (keyval)
         *keyval = 0;
     if (mods)
-        *mods = 0;
+        *mods = (GdkModifierType) 0;
 
     if (widget && GTK_WIDGET_REALIZED (widget))
         keymap = gdk_keymap_get_for_display (gtk_widget_get_display (widget));
@@ -431,10 +434,10 @@ moo_accel_translate_event (GtkWidget       *widget,
         keymap = gdk_keymap_get_default ();
 
     gdk_keymap_translate_keyboard_state (keymap, event->hardware_keycode,
-                                         event->state, event->group,
+                                         (GdkModifierType) event->state, event->group,
                                          keyval, NULL, NULL, &consumed);
     if (mods)
-        *mods = event->state & ~consumed & MOO_ACCEL_MODS_MASK;
+        *mods = (GdkModifierType) (event->state & ~consumed & MOO_ACCEL_MODS_MASK);
 }
 
 gboolean
@@ -529,7 +532,7 @@ parse_key (const char *string)
 static GdkModifierType
 parse_mod (const char *string, gssize len)
 {
-    GdkModifierType mod = 0;
+    GdkModifierType mod = (GdkModifierType) 0;
     char *stripped;
 
     stripped = g_strstrip (g_ascii_strdown (string, len));
@@ -543,10 +546,8 @@ parse_mod (const char *string, gssize len)
     else if (!strcmp (stripped, "cmd") ||
              !strcmp (stripped, "command"))
         mod = COMMAND_MASK;
-#if GTK_CHECK_VERSION(2,10,0)
     else if (!strcmp (stripped, "meta"))
         mod = GDK_META_MASK;
-#endif
     else if (!strncmp (stripped, "mod", 3) &&
              '1' <= stripped[3] && stripped[3] <= '5' && !stripped[4])
     {
@@ -564,6 +565,8 @@ parse_mod (const char *string, gssize len)
         mod = GDK_SHIFT_MASK;
     else if (!strcmp (stripped, "release"))
         mod = GDK_RELEASE_MASK;
+    else if (!strcmp (stripped, "primary"))
+        mod = COMMAND_MASK;
 
     g_free (stripped);
     return mod;
@@ -576,7 +579,7 @@ my_gtk_accelerator_parse (const char      *accel,
                           GdkModifierType *mods)
 {
     *key = 0;
-    *mods = 0;
+    *mods = (GdkModifierType) 0;
 
     while (*accel == '<')
     {
@@ -586,7 +589,7 @@ my_gtk_accelerator_parse (const char      *accel,
         if (!end || !(m = parse_mod (accel + 1, end - accel - 1)))
             return;
 
-        *mods |= m;
+        *mods = (GdkModifierType) (*mods | m);
         accel = end + 1;
     }
 
@@ -606,7 +609,7 @@ accel_parse_sep (const char     *accel,
 {
     char **pieces;
     guint n_pieces, i;
-    GdkModifierType mod = 0;
+    int mod = 0;
     guint key = 0;
 
     pieces = g_strsplit (accel, sep, 0);
@@ -634,7 +637,7 @@ out:
     if (keyval)
         *keyval = key;
     if (modifiers)
-        *modifiers = mod;
+        *modifiers = (GdkModifierType) mod;
     return key != 0;
 }
 
@@ -646,7 +649,7 @@ _moo_accel_parse (const char      *accel,
 {
     guint key = 0;
     guint len;
-    GdkModifierType mods = 0;
+    GdkModifierType mods = (GdkModifierType) 0;
     char *p;
 
     g_return_val_if_fail (accel && accel[0], FALSE);
@@ -702,10 +705,7 @@ _moo_accel_normalize (const char *accel)
 }
 
 
-#ifdef MOO_ENABLE_UNIT_TESTS
-
-#include <mooutils/mooutils-tests.h>
-#include <locale.h>
+#define TEST_PREFS_KEY_PREFIX "FooBar/"
 
 static void
 test_moo_accel_register (void)
@@ -724,8 +724,8 @@ test_moo_accel_register (void)
     };
 
     PA bad_accels[] = {
-        { "<Something>/Test1/a", NULL, "" },
-        { "<Something>/Test1/b", "<dd", "" },
+        { "<Something>/Foobar/aa", NULL, "" },
+        { "<Something>/Foobar/bb", "<dd", "" },
     };
 
     PA cases[] = {
@@ -780,7 +780,7 @@ test_moo_accel_register (void)
         const char *second_accel = cases[i].second_accel;
         const char *third_accel = cases[i].third_accel;
         guint key = 0;
-        GdkModifierType mods = 0;
+        GdkModifierType mods = (GdkModifierType) 0;
 
         TEST_EXPECT_WARNING (0, "_moo_accel_register and friends for path %s", path);
 
@@ -938,16 +938,46 @@ test_moo_get_accel_label (void)
     setlocale (LC_ALL, "");
 }
 
+static void
+delete_prefs_keys (void)
+{
+    GSList *keys = moo_prefs_list_keys (MOO_PREFS_RC);
+    while (keys)
+    {
+        char *key = keys->data;
+
+        if (g_str_has_prefix (key, "Shortcuts/Foobar/"))
+            moo_prefs_delete_key (key);
+
+        g_free (key);
+        keys = g_slist_delete_link (keys, keys);
+    }
+}
+
+static gboolean
+test_suite_init (void)
+{
+    delete_prefs_keys ();
+    return TRUE;
+}
+
+static void
+test_suite_cleanup (void)
+{
+    delete_prefs_keys ();
+}
+
 void
 moo_test_mooaccel (void)
 {
-    MooTestSuite *suite = moo_test_suite_new ("mooutils/mooaccel.c", NULL, NULL, NULL);
-    moo_test_suite_add_test (suite, "test of _moo_get_accel_label()",
+    MooTestSuite *suite = moo_test_suite_new ("mooaccel", "mooutils/mooaccel.c",
+                                              (MooTestSuiteInit) test_suite_init,
+                                              (MooTestSuiteCleanup) test_suite_cleanup,
+                                              NULL);
+    moo_test_suite_add_test (suite, "_moo_get_accel_label", "test of _moo_get_accel_label()",
                              (MooTestFunc) test_moo_get_accel_label, NULL);
-    moo_test_suite_add_test (suite, "test of _moo_accel_normalize()",
+    moo_test_suite_add_test (suite, "_moo_accel_normalize", "test of _moo_accel_normalize()",
                              (MooTestFunc) test_moo_accel_normalize, NULL);
-    moo_test_suite_add_test (suite, "test of _moo_accel_register() and friends",
+    moo_test_suite_add_test (suite, "_moo_accel_register", "test of _moo_accel_register() and friends",
                              (MooTestFunc) test_moo_accel_register, NULL);
 }
-
-#endif
