@@ -1,7 +1,7 @@
 /*
  *   moohelp.c
  *
- *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
  *   This file is part of medit.  medit is free software; you can
  *   redistribute it and/or modify it under the terms of the
@@ -13,7 +13,6 @@
  *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "config-dirs.h"
 #include "moohelp.h"
 #include "mooaccel.h"
 #include "mooutils-misc.h"
@@ -33,7 +32,7 @@ static MooHelpData *moo_help_get_data       (GtkWidget  *widget);
 static MooHelpData *
 moo_help_get_data (GtkWidget *widget)
 {
-    return g_object_get_data (G_OBJECT (widget), "moo-help-data");
+    return (MooHelpData*) g_object_get_data (G_OBJECT (widget), "moo-help-data");
 }
 
 
@@ -41,7 +40,7 @@ static gboolean
 moo_help_callback_id (GtkWidget *widget,
                       gpointer   data)
 {
-    moo_help_open_id (data, widget);
+    moo_help_open_id ((const char*) data, widget);
     return TRUE;
 }
 
@@ -181,7 +180,7 @@ find_help_dir (void)
 
     for (p = dirs; p && *p; ++p)
     {
-        char *d = g_build_filename (*p, "doc", g_get_prgname (), "help", NULL);
+        char *d = g_build_filename (*p, "doc", MOO_PACKAGE_NAME, "help", NULL);
 
         if (try_help_dir (d))
         {
@@ -203,7 +202,7 @@ warn_no_help (GtkWidget *parent)
     if (!been_here)
     {
         been_here = TRUE;
-        moo_error_dialog (parent, _("Help files not found"), NULL);
+        moo_error_dialog (_("Help files not found"), NULL, parent);
     }
 }
 
@@ -218,7 +217,7 @@ warn_no_help_file (const char *basename,
     msg = g_strdup_printf (_("File '%s' is missing in the directory '%s'"),
                            basename, dir_utf8);
 
-    moo_error_dialog (parent, _("Could not find help file"), msg);
+    moo_error_dialog (_("Could not find help file"), msg, parent);
 
     g_free (msg);
     g_free (dir_utf8);
@@ -268,8 +267,7 @@ open_html_file (const char *path,
 
     if (err)
     {
-        g_warning ("%s: error in g_spawn_async", G_STRLOC);
-        g_warning ("%s: %s", G_STRLOC, err->message);
+        g_warning ("error in g_spawn_async: %s", moo_error_message (err));
         g_error_free (err);
     }
 }
@@ -287,7 +285,8 @@ open_file_by_id (const char *id,
                  GtkWidget  *parent)
 {
     const char *dir;
-    char *filename, *basename;
+    const char *basename;
+    char *filename;
 
     g_return_if_fail (id != NULL);
 
@@ -297,10 +296,14 @@ open_file_by_id (const char *id,
         return;
     }
 
-    if (!strcmp (id, MOO_HELP_ID_CONTENTS))
+    if (strcmp (id, MOO_HELP_ID_CONTENTS) == 0)
         id = MOO_HELP_ID_INDEX;
 
-    basename = g_strdup_printf ("%s.html", id);
+    if (strcmp (id, MOO_HELP_ID_INDEX) == 0 || id[0] == 0)
+        basename = "index.html";
+    else
+        basename = id;
+
     filename = g_build_filename (dir, basename, NULL);
 
     if (g_file_test (filename, G_FILE_TEST_IS_REGULAR))
@@ -311,7 +314,6 @@ open_file_by_id (const char *id,
         warn_no_help_file (basename, parent);
 
     g_free (filename);
-    g_free (basename);
 }
 
 void
