@@ -1,7 +1,7 @@
 /*
  *   moohistorylist.c
  *
- *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
  *   This file is part of medit.  medit is free software; you can
  *   redistribute it and/or modify it under the terms of the
@@ -13,6 +13,10 @@
  *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+/**
+ * class:MooHistoryList: (parent GObject) (constructable) (moo.private 1)
+ **/
+
 #include "marshals.h"
 #include "mooutils/moohistorylist.h"
 #include "mooutils/mooprefs.h"
@@ -22,7 +26,7 @@
 
 #define MAX_NUM_HISTORY_ITEMS 10
 
-typedef MooHistoryItem Item;
+typedef MooHistoryListItem Item;
 
 struct _MooHistoryListPrivate {
     GtkTreeModel *model;
@@ -47,7 +51,7 @@ struct _MooHistoryListPrivate {
     gboolean allow_empty;
 };
 
-MOO_DEFINE_BOXED_TYPE_C (MooHistoryItem, moo_history_item)
+MOO_DEFINE_BOXED_TYPE_C (MooHistoryListItem, moo_history_list_item)
 
 static GHashTable *named_lists;
 
@@ -81,7 +85,7 @@ static void     _list_delete_last               (MooHistoryList     *list);
 static void     list_save_recent                (MooHistoryList     *list);
 
 static void     menu_item_activated             (MooHistoryList     *list,
-                                                 MooHistoryItem     *entry,
+                                                 MooHistoryListItem *entry,
                                                  gpointer            menu_data);
 
 
@@ -118,7 +122,7 @@ moo_history_list_class_init (MooHistoryListClass *klass)
                                              "user-id",
                                              "user-id",
                                              NULL,
-                                             G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+                                             (GParamFlags) (G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY)));
 
     g_object_class_install_property (gobject_class,
                                      PROP_MAX_ITEMS,
@@ -128,7 +132,7 @@ moo_history_list_class_init (MooHistoryListClass *klass)
                                              1,
                                              G_MAXUINT,
                                              MAX_NUM_HISTORY_ITEMS,
-                                             G_PARAM_READWRITE));
+                                             (GParamFlags) G_PARAM_READWRITE));
 
     g_object_class_install_property (gobject_class,
                                      PROP_EMPTY,
@@ -239,11 +243,11 @@ moo_history_list_finalize (GObject *object)
 }
 
 
-MooHistoryItem *
+MooHistoryListItem *
 moo_history_list_get_item (MooHistoryList *list,
                            GtkTreeIter    *iter)
 {
-    MooHistoryItem *item = NULL;
+    MooHistoryListItem *item = NULL;
     gtk_tree_model_get (list->priv->model, iter, 0, &item, -1);
     return item;
 }
@@ -266,11 +270,11 @@ moo_history_list_find (MooHistoryList *list,
 
             if (entry && list->priv->compare_func (text, entry, list->priv->compare_data))
             {
-                moo_history_item_free (entry);
+                moo_history_list_item_free (entry);
                 return TRUE;
             }
 
-            moo_history_item_free (entry);
+            moo_history_list_item_free (entry);
         }
         while (gtk_tree_model_iter_next (list->priv->model, iter));
     }
@@ -291,9 +295,9 @@ default_compare_func (const char *text,
 
 
 Item*
-moo_history_item_new (const char     *data,
-                      const char     *display,
-                      gboolean        builtin)
+moo_history_list_item_new (const char     *data,
+                           const char     *display,
+                           gboolean        builtin)
 {
     Item *item;
 
@@ -311,17 +315,17 @@ moo_history_item_new (const char     *data,
 
 
 Item*
-moo_history_item_copy (const Item *item)
+moo_history_list_item_copy (const Item *item)
 {
     g_return_val_if_fail (item != NULL, NULL);
-    return moo_history_item_new (item->data,
-                                 item->display,
-                                 item->builtin);
+    return moo_history_list_item_new (item->data,
+                                      item->display,
+                                      item->builtin);
 }
 
 
 void
-moo_history_item_free (Item *item)
+moo_history_list_item_free (Item *item)
 {
     if (item)
     {
@@ -373,7 +377,7 @@ moo_history_list_add_builtin (MooHistoryList *list,
 
     g_return_if_fail (MOO_IS_HISTORY_LIST (list));
 
-    new_item = moo_history_item_new (item, display_item, TRUE);
+    new_item = moo_history_list_item_new (item, display_item, TRUE);
     g_return_if_fail (new_item != NULL);
 
     if (moo_history_list_find (list, item, &iter))
@@ -381,7 +385,7 @@ moo_history_list_add_builtin (MooHistoryList *list,
         Item *old = moo_history_list_get_item (list, &iter);
         if (!old->builtin)
             _list_remove (list, &iter);
-        moo_history_item_free (old);
+        moo_history_list_item_free (old);
     }
 
     _list_insert (list, list->priv->num_builtin++, new_item);
@@ -471,11 +475,11 @@ moo_history_list_add_full (MooHistoryList *list,
             g_signal_emit (list, signals[CHANGED], 0);
         }
 
-        moo_history_item_free (old);
+        moo_history_list_item_free (old);
         return;
     }
 
-    new_entry = moo_history_item_new (entry, display_entry, FALSE);
+    new_entry = moo_history_list_item_new (entry, display_entry, FALSE);
     g_return_if_fail (new_entry != NULL);
 
     if (list->priv->num_builtin &&
@@ -503,7 +507,7 @@ moo_history_list_add_full (MooHistoryList *list,
     if (was_empty)
         g_object_notify (G_OBJECT (list), "empty");
 
-    moo_history_item_free (new_entry);
+    moo_history_list_item_free (new_entry);
 }
 
 
@@ -528,7 +532,7 @@ moo_history_list_get_last_item (MooHistoryList *list)
 
     data = item->data;
     item->data = NULL;
-    moo_history_item_free (item);
+    moo_history_list_item_free (item);
     return data;
 }
 
@@ -570,7 +574,7 @@ moo_history_list_remove (MooHistoryList *list,
     if (moo_history_list_is_empty (list))
         g_object_notify (G_OBJECT (list), "empty");
 
-    moo_history_item_free (item);
+    moo_history_list_item_free (item);
     return;
 }
 
@@ -694,13 +698,11 @@ moo_history_list_new (const char *user_id)
     MooHistoryList *list;
 
     if (user_id && named_lists)
-    {
-        list = g_hash_table_lookup (named_lists, user_id);
-        g_return_val_if_fail (list == NULL, NULL);
-    }
+        g_return_val_if_fail (g_hash_table_lookup (named_lists, user_id) == NULL, NULL);
 
-    list = g_object_new (MOO_TYPE_HISTORY_LIST,
-                         "user-id", user_id, NULL);
+    list = MOO_HISTORY_LIST (g_object_new (MOO_TYPE_HISTORY_LIST,
+                                           "user-id", user_id,
+                                           (const char*) NULL));
     if (user_id)
     {
         add_named_list (user_id, list);
@@ -721,7 +723,7 @@ moo_history_list_get (const char *user_id)
         named_lists = g_hash_table_new_full (g_str_hash, g_str_equal,
                                              g_free, g_object_unref);
 
-    list = g_hash_table_lookup (named_lists, user_id);
+    list = (MooHistoryList*) g_hash_table_lookup (named_lists, user_id);
 
     if (!list)
     {
@@ -746,9 +748,9 @@ moo_history_list_get_model (MooHistoryList *list)
  */
 
 static void
-menu_item_activated (MooHistoryList *list,
-                     MooHistoryItem *entry,
-                     gpointer        menu_data)
+menu_item_activated (MooHistoryList     *list,
+                     MooHistoryListItem *entry,
+                     gpointer            menu_data)
 {
     g_signal_emit (list, signals[ACTIVATE_ITEM], 0, entry, menu_data);
 }
@@ -796,8 +798,8 @@ _list_insert (MooHistoryList     *list,
         moo_menu_mgr_insert (list->priv->mgr,
                              NULL, index, NULL,
                              entry->display, tip, MOO_MENU_ITEM_ACTIVATABLE,
-                             moo_history_item_copy (entry),
-                             (GDestroyNotify) moo_history_item_free);
+                             moo_history_list_item_copy (entry),
+                             (GDestroyNotify) moo_history_list_item_free);
     }
     else
     {
@@ -835,15 +837,15 @@ _list_move_on_top (MooHistoryList     *list,
         moo_menu_mgr_insert (list->priv->mgr,
                              NULL, new_index, NULL,
                              entry->display, tip, MOO_MENU_ITEM_ACTIVATABLE,
-                             moo_history_item_copy (entry),
-                             (GDestroyNotify) moo_history_item_free);
+                             moo_history_list_item_copy (entry),
+                             (GDestroyNotify) moo_history_list_item_free);
     else
         moo_menu_mgr_insert_separator (list->priv->mgr, NULL, new_index);
 
     gtk_list_store_remove (list->priv->store, iter);
     gtk_list_store_insert (list->priv->store, iter, new_index);
     gtk_list_store_set (list->priv->store, iter, 0, entry, -1);
-    moo_history_item_free (entry);
+    moo_history_list_item_free (entry);
     g_free (tip);
 }
 
@@ -871,7 +873,7 @@ _list_delete_last (MooHistoryList *list)
 void
 _moo_history_list_load (MooHistoryList *list)
 {
-    MooMarkupDoc *xml;
+    MooMarkupNode *xml;
     MooMarkupNode *root, *node;
     char *root_path;
 
@@ -889,7 +891,7 @@ _moo_history_list_load (MooHistoryList *list)
     g_return_if_fail (xml != NULL);
 
     root_path = g_strdup_printf ("%s/" ELEMENT_RECENT_ITEMS, list->priv->user_id);
-    root = moo_markup_get_element (MOO_MARKUP_NODE (xml), root_path);
+    root = moo_markup_get_element (xml, root_path);
     g_free (root_path);
 
     if (!root)
@@ -908,7 +910,7 @@ _moo_history_list_load (MooHistoryList *list)
 
             if (!entry || !entry[0])
             {
-                g_warning ("%s: empty recent entry", G_STRLOC);
+                g_warning ("empty recent entry");
                 continue;
             }
 
@@ -916,7 +918,7 @@ _moo_history_list_load (MooHistoryList *list)
         }
         else
         {
-            g_warning ("%s: invalid '%s' element", G_STRLOC, node->name);
+            g_warning ("invalid '%s' element", node->name);
         }
     }
 
@@ -941,14 +943,14 @@ save_one (GtkTreeModel  *model,
     if (!item->builtin)
         moo_markup_create_text_element (root, ELEMENT_ITEM, item->data);
 
-    moo_history_item_free (item);
+    moo_history_list_item_free (item);
     return FALSE;
 }
 
 static void
 list_save_recent (MooHistoryList *list)
 {
-    MooMarkupDoc *xml;
+    MooMarkupNode *xml;
     MooMarkupNode *root;
     char *root_path;
 
@@ -962,7 +964,7 @@ list_save_recent (MooHistoryList *list)
     g_return_if_fail (xml != NULL);
 
     root_path = g_strdup_printf ("%s/" ELEMENT_RECENT_ITEMS, list->priv->user_id);
-    root = moo_markup_get_element (MOO_MARKUP_NODE (xml), root_path);
+    root = moo_markup_get_element (xml, root_path);
 
     if (root)
         moo_markup_delete_node (root);
@@ -973,7 +975,7 @@ list_save_recent (MooHistoryList *list)
         return;
     }
 
-    root = moo_markup_create_element (MOO_MARKUP_NODE (xml), root_path);
+    root = moo_markup_create_element (xml, root_path);
     g_return_if_fail (root != NULL);
 
     gtk_tree_model_foreach (GTK_TREE_MODEL (list->priv->store),
