@@ -1,7 +1,7 @@
 /*
  *   moowindow.c
  *
- *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
  *   This file is part of medit.  medit is free software; you can
  *   redistribute it and/or modify it under the terms of the
@@ -13,6 +13,10 @@
  *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+/**
+ * class:MooWindow: (parent GtkWindow)
+ **/
+
 #include "mooutils/moowindow.h"
 #include "mooutils/mooaction-private.h"
 #include "mooutils/mooactionbase-private.h"
@@ -20,7 +24,6 @@
 #include "mooutils/mooaccelprefs.h"
 #include "mooutils/mooaccel.h"
 #include "mooutils/mooprefs.h"
-#include "mooutils/moocompat.h"
 #include "marshals.h"
 #include "mooutils/moostock.h"
 #include "mooutils/mooactionfactory.h"
@@ -29,6 +32,8 @@
 #include "mooutils/mooutils-mem.h"
 #include "mooutils/mooeditops.h"
 #include "mooutils/mootype-macros.h"
+#include "mooutils/mooutils-enums.h"
+#include "mooutils/moocompat.h"
 #include <gtk/gtk.h>
 #include <gobject/gvaluecollector.h>
 
@@ -44,6 +49,7 @@
 
 #define TOOLBAR_STYLE_ACTION_ID "ToolbarStyle"
 
+static char *default_geometry = NULL;
 
 static GSList *window_instances = NULL;
 
@@ -149,6 +155,8 @@ static void         moo_window_set_statusbar_visible    (MooWindow      *window,
 static GtkAction   *create_toolbar_style_action         (MooWindow      *window,
                                                          gpointer        dummy);
 
+static MooCloseResponse moo_window_close_handler        (MooWindow      *window);
+
 static void         moo_window_set_focus                (GtkWindow      *window,
                                                          GtkWidget      *widget);
 static void         moo_window_disconnect_eo_widget     (MooWindow      *window);
@@ -191,7 +199,6 @@ enum {
 
 enum {
     CLOSE,
-    APPLY_PREFS,
     LAST_SIGNAL
 };
 
@@ -226,6 +233,8 @@ moo_window_class_init (MooWindowClass *klass)
     widget_class->key_press_event = moo_window_key_press_event;
     window_class->set_focus = moo_window_set_focus;
 
+    klass->close = moo_window_close_handler;
+
     moo_window_class_set_id (klass, "MooWindow", "Window");
 
     _moo_edit_ops_iface_install ();
@@ -338,11 +347,11 @@ moo_window_class_init (MooWindowClass *klass)
 
     g_object_class_install_property (gobject_class, PROP_MENUBAR_UI_NAME,
         g_param_spec_string ("menubar-ui-name", "menubar-ui-name", "menubar-ui-name",
-                             NULL, G_PARAM_READWRITE));
+                             NULL, (GParamFlags) G_PARAM_READWRITE));
 
     g_object_class_install_property (gobject_class, PROP_TOOLBAR_UI_NAME,
         g_param_spec_string ("toolbar-ui-name", "toolbar-ui-name", "toolbar-ui-name",
-                             NULL, G_PARAM_READWRITE));
+                             NULL, (GParamFlags) G_PARAM_READWRITE));
 
     g_object_class_install_property (gobject_class, PROP_ID,
         g_param_spec_string ("id", "id", "id", NULL, G_PARAM_READABLE));
@@ -353,7 +362,7 @@ moo_window_class_init (MooWindowClass *klass)
 
     g_object_class_install_property (gobject_class, PROP_UI_XML,
         g_param_spec_object ("ui-xml", "ui-xml", "ui-xml",
-                             MOO_TYPE_UI_XML, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+                             MOO_TYPE_UI_XML, (GParamFlags) (G_PARAM_READWRITE | G_PARAM_CONSTRUCT)));
 
     g_object_class_install_property (gobject_class, PROP_TOOLBAR,
         g_param_spec_object ("toolbar", "toolbar", "toolbar",
@@ -365,15 +374,15 @@ moo_window_class_init (MooWindowClass *klass)
 
     g_object_class_install_property (gobject_class, PROP_TOOLBAR_VISIBLE,
         g_param_spec_boolean ("toolbar-visible", "toolbar-visible", "toolbar-visible",
-                              TRUE, G_PARAM_READWRITE));
+                              TRUE, (GParamFlags) G_PARAM_READWRITE));
 
     g_object_class_install_property (gobject_class, PROP_MENUBAR_VISIBLE,
         g_param_spec_boolean ("menubar-visible", "menubar-visible", "menubar-visible",
-                              TRUE, G_PARAM_READWRITE));
+                              TRUE, (GParamFlags) G_PARAM_READWRITE));
 
     g_object_class_install_property (gobject_class, PROP_STATUSBAR_VISIBLE,
         g_param_spec_boolean ("statusbar-visible", "statusbar-visible", "statusbar-visible",
-                              TRUE, G_PARAM_READWRITE));
+                              TRUE, (GParamFlags) G_PARAM_READWRITE));
 
     INSTALL_PROP (PROP_MEO_CAN_COPY, "can-copy");
     INSTALL_PROP (PROP_MEO_CAN_CUT, "can-cut");
@@ -389,18 +398,10 @@ moo_window_class_init (MooWindowClass *klass)
                       G_OBJECT_CLASS_TYPE (klass),
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (MooWindowClass, close),
-                      g_signal_accumulator_true_handled, NULL,
-                      _moo_marshal_BOOL__VOID,
-                      G_TYPE_BOOLEAN, 0);
-
-    signals[APPLY_PREFS] =
-            g_signal_new ("apply-prefs",
-                      G_OBJECT_CLASS_TYPE (klass),
-                      G_SIGNAL_RUN_LAST,
-                      G_STRUCT_OFFSET (MooWindowClass, apply_prefs),
-                      NULL, NULL,
-                      _moo_marshal_VOID__VOID,
-                      G_TYPE_NONE, 0);
+                      moo_signal_accumulator_continue_cancel,
+                      GINT_TO_POINTER (MOO_CLOSE_RESPONSE_CONTINUE),
+                      _moo_marshal_ENUM__VOID,
+                      MOO_TYPE_CLOSE_RESPONSE, 0);
 }
 
 
@@ -454,7 +455,12 @@ moo_window_constructor (GType                  type,
     g_signal_connect (window, "notify::ui-xml",
                       G_CALLBACK (moo_window_update_ui), NULL);
 
-    if (moo_prefs_get_bool (setting (window, PREFS_REMEMBER_SIZE)))
+    if (default_geometry && *default_geometry)
+    {
+        if (!gtk_window_parse_geometry (GTK_WINDOW (window), default_geometry))
+            g_printerr (_("Could not parse geometry string '%s'\n"), default_geometry);
+    }
+    else if (moo_prefs_get_bool (setting (window, PREFS_REMEMBER_SIZE)))
     {
         int width = moo_prefs_get_int (setting (window, PREFS_WIDTH));
         int height = moo_prefs_get_int (setting (window, PREFS_HEIGHT));
@@ -492,6 +498,15 @@ moo_window_constructor (GType                  type,
 }
 
 
+void
+moo_window_set_default_geometry (const char *geometry)
+{
+    char *tmp = default_geometry;
+    default_geometry = g_strdup (geometry);
+    g_free (tmp);
+}
+
+
 static void
 parse_shadow_style (void)
 {
@@ -524,7 +539,7 @@ moo_window_init (MooWindow *window)
     window->status_area = gtk_hbox_new (FALSE, 0);
     window->statusbar = g_object_new (GTK_TYPE_STATUSBAR,
                                       "has-resize-grip", FALSE,
-                                      NULL);
+                                      (const char*) NULL);
     gtk_widget_set_name (GTK_WIDGET (window->statusbar), "no-shadow");
     gtk_box_pack_start (GTK_BOX (window->status_area),
                         GTK_WIDGET (window->statusbar),
@@ -535,7 +550,7 @@ moo_window_init (MooWindow *window)
 #else
                        "has-resize-grip", TRUE,
 #endif
-                       NULL);
+                       (const char*) NULL);
     gtk_widget_set_name (rg, "no-shadow");
     gtk_box_pack_end (GTK_BOX (window->status_area),
                       rg, FALSE, FALSE, 0);
@@ -607,9 +622,9 @@ static gboolean
 moo_window_delete_event (GtkWidget      *widget,
                          G_GNUC_UNUSED GdkEventAny *event)
 {
-    gboolean result = FALSE;
+    MooCloseResponse result = FALSE;
     g_signal_emit_by_name (widget, "close", &result);
-    return result;
+    return TRUE;
 }
 
 
@@ -618,7 +633,7 @@ accel_entry_new (guint            key,
                  GdkModifierType  mods,
                  GtkAction       *action)
 {
-    AccelEntry *entry = moo_new0 (AccelEntry);
+    AccelEntry *entry = g_slice_new0 (AccelEntry);
     entry->keyval = key;
     entry->modifiers = mods;
     entry->action = g_object_ref (action);
@@ -631,7 +646,7 @@ accel_entry_free (AccelEntry *entry)
     if (entry)
     {
         g_object_unref (entry->action);
-        moo_free (AccelEntry, entry);
+        g_slice_free (AccelEntry, entry);
     }
 }
 
@@ -690,7 +705,7 @@ accels_changed (MooWindow *window)
 {
     if (!window->priv->update_accels_idle)
         window->priv->update_accels_idle =
-            moo_idle_add ((GSourceFunc) update_accels, window);
+            gdk_threads_add_idle ((GSourceFunc) update_accels, window);
 }
 
 static gboolean
@@ -777,33 +792,33 @@ moo_window_save_size (MooWindow *window)
 {
     if (!window->priv->save_size_id)
         window->priv->save_size_id =
-                moo_idle_add ((GSourceFunc)save_size, window);
+                gdk_threads_add_idle ((GSourceFunc)save_size, window);
     return FALSE;
 }
 
 
+static MooCloseResponse
+moo_window_close_handler (G_GNUC_UNUSED MooWindow *window)
+{
+    return MOO_CLOSE_RESPONSE_CONTINUE;
+}
+
 gboolean
 moo_window_close (MooWindow *window)
 {
-    gboolean result = FALSE;
-    g_signal_emit_by_name (window, "close", &result);
-    if (!result)
+    MooCloseResponse response = MOO_CLOSE_RESPONSE_CONTINUE;
+
+    g_signal_emit_by_name (window, "close", &response);
+
+    if (response == MOO_CLOSE_RESPONSE_CANCEL)
+    {
+        return FALSE;
+    }
+    else
     {
         gtk_widget_destroy (GTK_WIDGET (window));
         return TRUE;
     }
-    else
-    {
-        return FALSE;
-    }
-}
-
-
-void
-moo_window_apply_prefs (MooWindow *window)
-{
-    g_return_if_fail (MOO_IS_WINDOW (window));
-    g_signal_emit (window, signals[APPLY_PREFS], 0);
 }
 
 
@@ -833,13 +848,13 @@ moo_window_set_property (GObject      *object,
     {
         case PROP_TOOLBAR_UI_NAME:
             name = g_value_get_string (value);
-            MOO_ASSIGN_STRING (window->priv->toolbar_ui_name, name ? name : "");
+            MOO_ASSIGN_STRING (window->priv->toolbar_ui_name, MOO_NZS (name));
             g_object_notify (object, "toolbar-ui-name");
             break;
 
         case PROP_MENUBAR_UI_NAME:
             name = g_value_get_string (value);
-            MOO_ASSIGN_STRING (window->priv->menubar_ui_name, name ? name : "");
+            MOO_ASSIGN_STRING (window->priv->menubar_ui_name, MOO_NZS (name));
             g_object_notify (object, "menubar-ui-name");
             break;
 
@@ -960,6 +975,21 @@ moo_window_get_property (GObject      *object,
 }
 
 
+void
+moo_window_set_title (MooWindow  *window,
+                      const char *title)
+{
+    const char *old_title;
+
+    g_return_if_fail (window != NULL);
+
+    old_title = gtk_window_get_title (GTK_WINDOW (window));
+
+    if (!moo_str_equal (title, old_title))
+        gtk_window_set_title (GTK_WINDOW (window), MOO_NZS (title));
+}
+
+
 static void
 moo_window_update_toolbar (MooWindow *window)
 {
@@ -1134,7 +1164,7 @@ toolbar_style_toggled (MooWindow            *window,
     GtkToolbarStyle style = GPOINTER_TO_INT (data);
     if (window->toolbar)
         gtk_toolbar_set_style (GTK_TOOLBAR (window->toolbar), style);
-    moo_prefs_set_enum (setting (window, PREFS_TOOLBAR_STYLE), style);
+    moo_prefs_set_int (setting (window, PREFS_TOOLBAR_STYLE), style);
 }
 
 
@@ -1211,9 +1241,8 @@ init_prefs (MooWindow *window)
     moo_prefs_new_key_bool (setting (window, PREFS_SHOW_TOOLBAR), TRUE);
     moo_prefs_new_key_bool (setting (window, PREFS_SHOW_MENUBAR), TRUE);
     moo_prefs_new_key_bool (setting (window, PREFS_SHOW_STATUSBAR), TRUE);
-    moo_prefs_new_key_enum (setting (window, PREFS_TOOLBAR_STYLE),
-                            GTK_TYPE_TOOLBAR_STYLE,
-                            get_toolbar_style_gtk (window));
+    moo_prefs_new_key_int (setting (window, PREFS_TOOLBAR_STYLE),
+                           get_toolbar_style_gtk (window));
 
     moo_prefs_create_key (setting (window, PREFS_MAXIMIZED), MOO_PREFS_STATE, G_TYPE_BOOLEAN, FALSE);
     moo_prefs_create_key (setting (window, PREFS_WIDTH), MOO_PREFS_STATE, G_TYPE_INT, 800);
@@ -1223,7 +1252,7 @@ init_prefs (MooWindow *window)
 static GtkToolbarStyle
 get_toolbar_style (MooWindow *window)
 {
-    GtkToolbarStyle s = moo_prefs_get_enum (setting (window, PREFS_TOOLBAR_STYLE));
+    GtkToolbarStyle s = moo_prefs_get_int (setting (window, PREFS_TOOLBAR_STYLE));
     g_return_val_if_fail (s < N_STYLES, GTK_TOOLBAR_ICONS);
     return s;
 }
@@ -1563,6 +1592,12 @@ moo_window_class_new_action_custom (MooWindowClass     *klass,
 }
 
 
+/**
+ * moo_window_class_find_action: (moo.lua 0) (moo.private 1)
+ *
+ * @klass: (type MooWindowClass*):
+ * @id: (type const-utf8):
+ **/
 gboolean
 moo_window_class_find_action (MooWindowClass *klass,
                               const char     *id)
@@ -1580,6 +1615,12 @@ moo_window_class_find_action (MooWindowClass *klass,
 }
 
 
+/**
+ * moo_window_class_remove_action: (moo.lua 0) (moo.private 1)
+ *
+ * @klass: (type MooWindowClass*):
+ * @id: (type const-utf8):
+ **/
 void
 moo_window_class_remove_action (MooWindowClass     *klass,
                                 const char         *action_id)
@@ -2041,7 +2082,7 @@ collect_params_and_props (guint              n_callback_args,
 
         if (!action_factory)
         {
-            g_warning ("%s: error in moo_action_factory_new_a()", G_STRLOC);
+            g_warning ("oops");
             goto error;
         }
 
@@ -2123,7 +2164,7 @@ moo_window_class_new_action (MooWindowClass     *klass,
     if (!success)
     {
         if (error)
-            g_critical ("%s: %s", G_STRLOC, error);
+            g_critical ("%s", error);
         g_free (error);
         return;
     }
@@ -2161,7 +2202,7 @@ _moo_window_class_new_action_callback (MooWindowClass *klass,
     if (!success)
     {
         if (error)
-            g_critical ("%s: %s", G_STRLOC, error);
+            g_critical ("%s", error);
         g_free (error);
         return;
     }
